@@ -16,7 +16,15 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+ 
+typedef struct {
 
+  gchar *name,*command,*info;
+  gint  time;
+  gboolean iscountdown;
+  gpointer pd;
+  
+} alarm_t;
 
 typedef struct {
   GtkWidget 	*eventbox,		/* Main container widget
@@ -28,13 +36,16 @@ typedef struct {
 		*buttonup, *buttondown,
 		*spin_repeat, *spin_interval,			/* spinbuttons for alarm repeat */
 		*menu,
+		*glob_command_entry,		/* Text entry widget for
+								the default alarm command */
+		*global_command_box,	/* Box holding the default
+								command settings */
 		*repeat_alarm_box;		/* Box holding the repeat 
 								alarm settings */
 		
   XfcePanelPlugin *base;			/* The plugin widget */
-  GtkListStore *list;				/* The alarms list */
-  gint 		count,					/* Number of alarms */
-			selected,
+  GtkListStore *liststore;				/* The alarms list */
+  gint 		count,
 			timeout_period_in_sec,	/* Active countdown period */
 			repetitions,			/* Number of alarm repeats */
 			rem_repetitions,		/* Remaining repeats */
@@ -47,19 +58,22 @@ typedef struct {
 									alarm command is set */
 			selecting_starts,       /* selecting a timer also starts it */							   
 			repeat_alarm,			/* Repeat alarm */
+			use_global_command,		/* Use a default alarm command
+									if no alarm command is set */
 			alarm_repeating,		/* True while alarm repeats */
 			is_paused,				/* True if the countdown is paused */
 			is_countdown;			/* True if the alarm type is contdown */
   GtkTooltips 	*tip;				/* Tooltip for panel */
-  GArray 	*menuarray;			/* Array of popup 
-								menuitems (alarms). These 
-								are used to find the index
-								of the selected menuitem. */
   gchar 	*timeout_command,	/* Command when countdown ends */
+			*global_command,	/* The global (default) command to be
+								run when countdown ends */
+			*active_timer_name, /* Name of the timer running */
   			*configfile;		/* Full address of the permanent
   								config file -- this is not the
   								plugin rc file. */
-  GTimer 	*timer;				/* Keeps time elapsed */
+  GTimer 	*timer;				/* Keeps track of the time elapsed */
+  GList		*alarm_list,		/* List of alarms */
+  			*selected;			/* Selected alarm */
 
 } plugin_data;
 
@@ -73,3 +87,4 @@ typedef struct {
   GtkWindow 	*window;				/* Add/Edit window */
   plugin_data 	*pd;					/* Plugin data */
 } alarm_data;
+
